package org.rcisoft.sys.sysuser.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.service.CyBaseService;

import org.rcisoft.sys.sysuser.dao.SysDeptRepositorys;
import org.rcisoft.sys.sysuser.entity.SysDept;
import org.rcisoft.sys.sysuser.service.SysDeptService;
import org.rcisoft.sys.wbac.dept.dao.SysDeptRepository;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年11月18日 上午9:03:23.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysDeptServiceImpl extends ServiceImpl<SysDeptRepositorys, SysDept>  implements SysDeptService {


    /**
    * 保存 部门表
    * @param sysDept
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysDept sysDept){
        //增加操作
        int line = baseMapper.insert(sysDept);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            sysDept.getBusinessId()+"的部门表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 部门表
    * @param sysDept
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysDept sysDept){
        int line = baseMapper.realDelete(sysDept);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            sysDept.getBusinessId()+"的部门表信息");
        return new CyPersistModel(line);
    }
    /**
    * 逻辑删除 部门表
    * @param sysDept
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysDept sysDept){
        sysDept.setDeleted();
        int line = baseMapper.deleteById(sysDept);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                sysDept.getBusinessId()+"的部门表信息");
            return new CyPersistModel(line);
        }
    /**
    * 修改 部门表
    * @param sysDept
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysDept sysDept){
        int line = baseMapper.updateById(sysDept);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ sysDept.getBusinessId()+"的部门表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 部门表
    * @param id
    * @return
    */
    @Override
    public SysDept findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 部门表
    * @param sysDept
    * @return
    */
    @Override
    public IPage<SysDept> findAllByPagination(CyPageInfo<SysDept> paginationUtility,
        SysDept sysDept){
        return baseMapper.querySysDeptsPaged(paginationUtility,sysDept);

    }
    /**
    * 查询list 部门表
    * @param sysDept
    * @return
    */
    @Override
    public List<SysDept> findAll(SysDept sysDept){
        return baseMapper.querySysDepts(sysDept);

    }

    /**
    * 导出部门表
    * @return
    */
    @Override
    public List<SysDept> export(SysDept sysDept) {
        List<SysDept> sysDeptList = baseMapper.querySysDepts(sysDept);
        return sysDeptList;
    }

}
