package org.rcisoft.sys.sysuser.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.common.component.CyGlobal;
import org.rcisoft.core.component.CyDownErrorTxtComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.result.enums.CyResSvcExcEnum;
import org.rcisoft.core.util.CyCompareToolUtils;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;

import org.rcisoft.sys.constant.CyDictCons;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictionaryService;
import org.rcisoft.sys.rbac.user.dto.ImportUserDTO;
import org.rcisoft.sys.sysuser.dao.SysDeptRepositorys;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.dto.ExportUserDTO;
import org.rcisoft.sys.sysuser.entity.SysDept;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.SysUserService;
import org.rcisoft.sys.sysuser.util.CheckUtil;
import org.rcisoft.sys.wbac.user.dto.ErrorDTO;
import org.rcisoft.sys.wbac.user.dto.ExcelDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.lang.reflect.Field;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by cy on 2022年11月16日 下午3:19:19.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysUserServiceImpl extends ServiceImpl<SysUserRepositorys, SysUser> implements SysUserService {
    @Autowired
    private SysUserRepositorys sysUserRepositorys;
    @Autowired
    private SysDeptRepositorys sysDeptRepositorys;
    @Autowired
    private SysDictDataRepositorys sysDictDataRepositorys;
    @Autowired
    private DictionaryService dictionaryService;
    @Autowired
    private CyDownErrorTxtComp errorTxt;
    @Autowired
    private CyGlobal global;

    @Autowired
    private PasswordEncoder passwordEncoder;

    /**
     * 保存 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysUser sysUser) {
        //增加操作
        sysUser.setUsername(sysUser.getPhone());
        int line = sysUserRepositorys.insert(sysUser);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysUser sysUser) {
        int line = sysUserRepositorys.delSysUser(Long.valueOf(sysUser.getBusinessId()));
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysUser sysUser) {
        sysUser.setDeleted();
        int line = baseMapper.deleteById(sysUser);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysUser sysUser) {
        int line = sysUserRepositorys.updateSysUser(sysUser);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 用户表
     *
     * @param businessId
     * @return
     */
    @Override
    public SysUser findById(Long businessId) {
        return sysUserRepositorys.querySysUsersById(businessId);
    }

    /**
     * 分页查询 用户表
     *
     * @param sysUser
     * @return
     */
    @Override
    public IPage<SysUser> findAllByPagination(CyPageInfo<SysUser> paginationUtility, SysUser sysUser) {
        IPage<SysUser> sysUserIPage = baseMapper.querySysUsersPaged(paginationUtility, sysUser);
        for (int i = 0; i < sysUserIPage.getRecords().size(); i++) {
            BigInteger deptId = sysUserIPage.getRecords().get(i).getDeptId();
            //根据部门id查询部门表
            List<SysDept> sysDepts = sysDeptRepositorys.queryDeptById(deptId);
            if (sysDepts.size() != 0) {
                //获取部门名称
                String deptName = sysDepts.get(0).getDeptName();
                sysUserIPage.getRecords().get(i).setDeptName(deptName);
            }

            //查询职位id
            String position = sysUserIPage.getRecords().get(i).getPosition();
            List<SysDictData> sysDictData = sysDictDataRepositorys.queryDataByPosition(position);
            if (sysDictData.size() != 0) {
                //查询职位中文名
                String dictLabel = sysDictData.get(0).getDictLabel();
                sysUserIPage.getRecords().get(i).setDictLabelByPosition(dictLabel);
            }

            //查询班次id
            String nltBz = sysUserIPage.getRecords().get(i).getNltBz();
            List<SysDictData> sysDictData1 = sysDictDataRepositorys.queryDataByNltBz(nltBz);
            if (sysDictData1.size() != 0) {
                //查询班次中文名
                String dictLabel = sysDictData1.get(0).getDictLabel();
                sysUserIPage.getRecords().get(i).setDictLabelBynltBz(dictLabel);
            }

        }
        return sysUserIPage;

    }

    /**
     * 查询list 用户表
     *
     * @param sysUser
     * @return
     */
    @Override
    public List<SysUser> findAll(SysUser sysUser) {
        return baseMapper.querySysUsers(sysUser);

    }

    public List<SysUser> findAllByData(SysUser sysUser) {
        return sysDictDataRepositorys.queryData(sysUser);

    }

    public List<SysUser> findAllByDept(SysUser sysUser) {
        return sysDeptRepositorys.queryDept(sysUser);

    }

    /**
     * 导出用户表
     *
     * @return
     */
    @Override
    public List<SysUser> export(SysUser sysUser) {
        List<SysUser> sysUserList = baseMapper.querySysUsers(sysUser);
        return sysUserList;
    }

    @Override
    public int exportEmptyTemplate(HttpServletResponse response) {
        List<ExportUserDTO> imOutSysUsers = new ArrayList<>();
        ExportUserDTO exportUserDTO = new ExportUserDTO();
        for (int i = 0; i < 500; i++) {
            imOutSysUsers.add(exportUserDTO);
        }
        //1.用户性别
/*        List<DictData> userSex = dictionaryService.selectByTypes(CyDictCons.USER_SEX);
        String[] userSexs = new String[userSex.size()];
        for (int i = 0; i <= userSex.size() - 1; i++) {
            userSexs[i] = (String) userSex.get(i).getDictLabel();
        }*/
        //2.岗位
       /* SysPostDTO sysPostDTO = new SysPostDTO();
        List<SysPost> posts = sysPostService.findAll(sysPostDTO);
        String[] postss = new String[posts.size() - 1];
        for (int i = 1; i <= posts.size() - 1; i++) {
            postss[i - 1] = (String) posts.get(i).getPostName() + "/" + posts.get(i).getDeptId();
        }
*/
        ExcelDTO excelDto = new ExcelDTO();
       // excelDto.setSex(userSexs);
        // excelDto.setPost(postss);

        CyEpExcelUtil.exportExcelSelect(imOutSysUsers, "用户信息", "用户",
                ExportUserDTO.class, "模板.xlsx", response, excelDto);
        return 1;
    }

    void errorText(String error, ImportUserDTO importUserDTO, List<Map<Integer, List<String>>> maps, String fileName) {
        List<String> strings = new ArrayList<String>();
        strings.add(error);
        Map<Integer, List<String>> map = new HashMap<>();
        map.put(-1, strings);
        maps.add(map);
        importUserDTO.setFilename(fileName);
    }

    @Override
    @Transactional
    public ImportUserDTO importUserExcel(MultipartFile file) {
        ImportUserDTO importUserDTO = new ImportUserDTO();
        Calendar calendar = Calendar.getInstance();
        String nowTime = new SimpleDateFormat("yyyyMMddHHmmss").format(calendar.getTime());
        String fileName = "用户错误信息" + nowTime + ".txt";
        List<Map<Integer, List<String>>> maps = new ArrayList<>();
        ExportUserDTO exportUserDTO = new ExportUserDTO();
        boolean flag = false;
        try {
            flag = CyEpExcelUtil.isFileInUser(file, exportUserDTO, 2);
        } catch (Exception e) {
            throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
        }
        if (!flag || !CyCompareToolUtils.compareFileName(file, "用户信息", 0, 0)) {
            String errorBefore = "上传模板错误请重新下载！";
            errorText(errorBefore, importUserDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        }
        /** 从excel表中读到数据，有一部分为空值 */
        List<ExportUserDTO> sysUserList = CyEpExcelUtil.importExcel(file, 2, 1, ExportUserDTO.class);
        /** 创建用户的集合对象 */
        List<SysUser> sysUserInsert = new ArrayList<>();
        /** 遍历 从Excel表中读到的数据 抛出不合适的数据 */
        //判断整个Excel是不是为空 计算非空数据有多少条
        int information = CyCompareToolUtils.cmpareInformation(sysUserList);
        if (information == 0) {
            String errorBefore = "Excel文件不可为空，请检查Excel文件是否填写！";
            errorText(errorBefore, importUserDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        } else if (information > 500) {
            String errorMore = "单次导入数据不可超过500，请检查！";
            errorText(errorMore, importUserDTO, maps, fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        } else {
            //去除空数据
            try {
                sysUserList = ThrowNull(sysUserList);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            //校验 将错误信息放入maps
            int index = 3; //从第3行开始
            SysUser sysUserDTO = new SysUser();
            sysUserDTO.setDeleted();
            List<SysUser> sysUsers = sysUserRepositorys.querySysUsers(sysUserDTO);
            for (ExportUserDTO sysUser : sysUserList) {
                //判断 是否为空
                ErrorDTO userNull = CheckUtil.userNotNull(sysUser, index);
                if (userNull.getSuccess() == 1) {
                    maps.add(userNull.getMap());
                }
                //判断 信息是否重复
                ErrorDTO userRepeat = CheckUtil.userRepeat(sysUser, index, sysUsers);
                if (userRepeat.getSuccess() == 1) {
                    maps.add(userRepeat.getMap());
                }
                index++;
            }
        }
        if (null != maps && maps.size() > 0) {
            importUserDTO.setFilename(fileName);
            try {
                errorTxt.exportTxtPaper(maps, fileName);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            return importUserDTO;
        }
        /** 将 信息插入sys_user  */
        if (sysUserList.size() > 0) {
            //用户性别数据字典
            List<DictData> userSex = dictionaryService.selectByTypes(CyDictCons.USER_SEX);
            for (ExportUserDTO sysUser : sysUserList) {
                //数据库需要插入的用户信息
                SysUser userInsert = new SysUser();
                BeanUtil.copyProperties(sysUser, userInsert);
                userInsert.setPassword(passwordEncoder.encode(global.getResetPassword()));
                userInsert.setUserType("1");
                for (DictData dictData : userSex) {
                    if (dictData.getDictLabel().equals(userInsert.getSex())) {
                        userInsert.setSex(dictData.getDictValue());
                    }
                }
                sysUserInsert.add(userInsert);
            }
            //插入 用户信息
            saveBatch(sysUserInsert);
/*            //查询岗位信息
            List<SysPost> sysPosts = sysPostRepository.querySysPosts(new SysPostDTO());
            //插入 用户-岗位信息
            for (ExportUserDTO sysUser : sysUserList) {
                List<org.rcisoft.sys.wbac.user.entity.SysUser> userId = sysUserRepository.queryUserByUserName(sysUser.getUsername());
                //获取岗位
                String postNames = sysUser.getPostIds();
                String[] postName = postNames.split(",");
                List<SysUserPost> sysUserPosts = new ArrayList<>();
                SysUserPost addUserPost;
                for (SysPost sysPost : sysPosts) {
                    for (String post : postName) {
                        String postNam = post.split("/")[0];//获取岗位名称
                        String postDeptId = post.split("/")[1];//获取岗位所属部门ID
                        if (sysPost.getPostName().equals(postNam)) {
                            if (String.valueOf(sysPost.getDeptId()).equals(postDeptId)) {
                                addUserPost = new SysUserPost();
                                addUserPost.setPostId(sysPost.getBusinessId());
                                addUserPost.setUserId(userId.get(0).getBusinessId());
                                sysUserPosts.add(addUserPost);
                            }
                        }
                    }
                }
                //插入人对应的岗位
                if (!CollectionUtils.isEmpty(sysUserPosts)) {
                    sysUserPostRepository.insertUserPost(sysUserPosts);
                }
            }*/
        }
        return importUserDTO;
    }

    /**
     * 去除List中空数据
     *
     * @param list
     * @param <T>
     * @return
     * @throws Exception
     */
    public <T> List<T> ThrowNull(List<T> list) throws Exception {
        List<T> list2 = new ArrayList<>();
        if (list != null) {
            for (T t : list) {
                if (outNull(t) == true) {
                    list2.add(t);
                }
            }
        }
        return list2;
    }

    //只要有一个不为null则添加进去
    public boolean outNull(Object o) throws Exception {
        if (o != null) {
            Class aa = o.getClass();
            Field[] fields = aa.getDeclaredFields();
            for (Field field : fields) {//id
                field.setAccessible(true);
                if (field.get(o) != null && !field.getName().equals("rowNum")) {
                    return true;
                }
            }
        }
        return false;
    }

}
