package org.rcisoft.sys.sysuser.util;

import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.constant.CySysCons;
import org.rcisoft.sys.sysuser.dto.ExportUserDTO;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.wbac.user.dto.ErrorDTO;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@ConditionalOnProperty(prefix = "cy.model", name = "permission", havingValue = CySysCons.WORK_BASE)
public class CheckUtil {


    /**
     * 对导入的用户信息进行非空校验
     *
     * @param sysUser 用户信息
     * @param row     行数
     * @return
     */
    public static ErrorDTO userNotNull(ExportUserDTO sysUser, int row) {
        ErrorDTO errorDTO = new ErrorDTO();
        Map<Integer, List<String>> notTrues = new HashMap<>();
        List<String> notTrue = new ArrayList<>();
        errorDTO.setSuccess(0);
        if (StringUtils.isEmpty(sysUser.getUsername())) {
            notTrue.add("用户名不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getName())) {
            notTrue.add("姓名不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getEmail())) {
            notTrue.add("邮箱不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getPhone())) {
            notTrue.add("手机号不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getIdNumber())) {
            notTrue.add("身份证号不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getSex())) {
            notTrue.add("性别不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getPostIds())) {
            notTrue.add("岗位不能为空");
        }
        if (notTrue.size() > 0) {
            errorDTO.setSuccess(1);
            notTrues.put(row, notTrue);
            errorDTO.setMap(notTrues);
        }
        return errorDTO;
    }

    /**
     * 对导入信息进行查重校验
     *
     * @param sysUser  导入用户信息
     * @param row      行数
     * @param sysUsers 已有用户信息
     * @return
     */
    public static ErrorDTO userRepeat(ExportUserDTO sysUser, int row, List<SysUser> sysUsers) {
        ErrorDTO errorDTO = new ErrorDTO();
        Map<Integer, List<String>> notTrues = new HashMap<>();
        List<String> notTrue = new ArrayList<>();
        errorDTO.setSuccess(0);
        for (SysUser existenceUser : sysUsers) {
            if (existenceUser.getUsername().equals(sysUser.getUsername())) {
                notTrue.add("登录名不能重复");
            }
            if (existenceUser.getIdNumber().equals(sysUser.getIdNumber())) {
                notTrue.add("身份证号不能重复");
            }
        }
        if (notTrue.size() > 0) {
            errorDTO.setSuccess(1);
            notTrues.put(row, notTrue);
            errorDTO.setMap(notTrues);
        }
        return errorDTO;
    }

}
