package org.rcisoft.sys.visit.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.service.VisitInfomationService;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


import java.util.List;

/**
 * Created by cy on 2022年11月16日 下午2:40:21.
 */
@RestController
@RequestMapping("/visitInfoMation")
public class VisitInfomationController extends CyPaginationController<VisitInfoMation> {


    @Autowired
    private VisitInfomationService visitInfomationServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:add')")
    @CyOpeLogAnno(title = "system-邀访信息管理-新增邀访信息", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加邀访信息", notes = "添加邀访信息")
    @PostMapping(value = "/add")
    public CyResult add(@RequestBody VisitInfoMation visitInfomation) {
        CyPersistModel data = visitInfomationServiceImpl.persist(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:delete')")
    @CyOpeLogAnno(title = "system-邀访信息管理-删除邀访信息", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除邀访信息", notes = "逻辑删除邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, VisitInfoMation visitInfomation) {
        visitInfomation.setBusinessId(businessId);
        CyPersistModel data = visitInfomationServiceImpl.removeLogical(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:delete')")
    @CyOpeLogAnno(title = "system-邀访信息管理-删除邀访信息", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除邀访信息", notes = "删除邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, VisitInfoMation visitInfomation) {
        visitInfomation.setBusinessId(businessId);
        CyPersistModel data = visitInfomationServiceImpl.remove(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:update')")
    @CyOpeLogAnno(title = "system-邀访信息管理-修改邀访信息", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改邀访信息", notes = "修改邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody VisitInfoMation visitInfomation, BindingResult bindingResult) {
        visitInfomation.setBusinessId(visitInfomation.getBusinessId());
        CyPersistModel data = visitInfomationServiceImpl.merge(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:query')")
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一邀访信息", notes = "查询单一邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:list')")
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询邀访信息集合", notes = "查询邀访信息集合")
    @GetMapping(value = "/queryVisitInfomations")
    public CyResult queryVisitInfomations(VisitInfoMation visitInfomation) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.findAll(visitInfomation));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:list')")
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询邀访信息集合", notes = "分页查询邀访信息集合")
    @GetMapping(value = "/queryVisitInfomationByPagination")
    public CyGridModel listByPagination(VisitInfoMation visitInfomation) {
        visitInfomationServiceImpl.findAllByPagination(getPaginationUtility(), visitInfomation);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出邀访信息信息", notes = "导出邀访信息信息")
    @GetMapping(value = "/export")
    public CyResult outVisitInfomation(HttpServletResponse response, VisitInfoMation visitInfomation, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "邀访信息信息.xls";
                break;
            case "1":
                excelName = "邀访信息信息.xlsx";
                break;
            case "2":
                excelName = "邀访信息信息.csv";
                break;
        }
        List<VisitInfoMation> visitInfomationList = visitInfomationServiceImpl.export(visitInfomation);
        CyEpExcelUtil.exportExcel(visitInfomationList, "邀访信息信息", "邀访信息信息", VisitInfoMation.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationList);
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-查询最近受邀记录", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询最近受邀记录", notes = "查询最近受邀记录")
    @GetMapping("/lastDetail")
    public CyResult lastDetail(HttpServletRequest request) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.lastDetail(request));
    }

}
