package org.rcisoft.sys.visit.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.SysUserService;
import org.rcisoft.sys.visit.entity.VisitPerson;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
* Created by cy on 2022年11月16日 下午3:19:19.
*/
@RestController
@RequestMapping("/visitPersonManage")
public class VisitPersonController extends CyPaginationController<SysUser> {


    @Autowired
    private SysUserService sysUserServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:add')")
    @CyOpeLogAnno(title = "system-访客表管理-新增访客表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加访客表", notes="添加访客表")
    @PostMapping(value = "/add")
    public CyResult add(@RequestBody SysUser sysUser, BindingResult bindingResult) {
        sysUser.setUserType("1");
        CyPersistModel data = sysUserServiceImpl.persist(sysUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUser);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:user:delete')")
    @CyOpeLogAnno(title = "system-访客表管理-删除访客表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除访客表", notes="逻辑删除访客表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId,SysUser sysUser) {
        sysUser.setBusinessId(businessId);
        CyPersistModel data = sysUserServiceImpl.removeLogical(sysUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //逻辑删除
    //@PreAuthorize("@cyPerm.hasPerm('sys:user:delete')")
    @CyOpeLogAnno(title = "system-访客表管理-删除访客表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除访客表", notes="删除访客表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,SysUser sysUser) {
        sysUser.setBusinessId(businessId);
        CyPersistModel data = sysUserServiceImpl.remove(sysUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:update')")
    @CyOpeLogAnno(title = "system-访客表管理-修改访客表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改访客表", notes="修改访客表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody SysUser sysUser, BindingResult bindingResult) {
        CyPersistModel data = sysUserServiceImpl.merge(sysUser);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUser);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:query')")
    @CyOpeLogAnno(title = "system-访客表管理-查询访客表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一访客表", notes="查询单一访客表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Long businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUserServiceImpl.findById(businessId));
    }

//    //@PreAuthorize("@cyPerm.hasPerm('sys:user:list')")
    @CyOpeLogAnno(title = "system-访客表管理-查询访客表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询访客表集合", notes="查询访客表集合")
    @GetMapping(value = "/querySysUsers")
    public CyResult querySysUsers(SysUser sysUser) {
        sysUser.setUserType("1");
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysUserServiceImpl.findAll(sysUser));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:list')")
    @CyOpeLogAnno(title = "system-访客表管理-查询访客表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询访客表集合", notes="分页查询访客表集合")
    @GetMapping(value = "/querySysUserByPagination")
    public CyGridModel listByPagination(SysUser sysUser) {
        sysUser.setUserType("1");
        sysUserServiceImpl.findAllByPagination(getPaginationUtility(), sysUser);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-访客表管理-查询访客表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出访客表信息", notes = "导出访客表信息")
    @GetMapping(value = "/export")
    public CyResult outSysUser(HttpServletResponse response,SysUser sysUser,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        sysUser.setUserType("1");
        String excelName="";
        switch(excelId){
            case "0": excelName="访客表信息.xls";break;
            case "1": excelName="访客表信息.xlsx";break;
            case "2": excelName="访客表信息.csv";break;
        }
        List<SysUser> sysUserList = sysUserServiceImpl.export(sysUser);
        List<VisitPerson> visitPersonList = new ArrayList<>();
        for (SysUser u: sysUserList) {
            VisitPerson visitPerson = new VisitPerson();
            visitPerson.setName(u.getName());
            visitPerson.setPhone(u.getPhone());
            visitPerson.setBlacklistFlag(u.getBlacklistFlag());
            visitPersonList.add(visitPerson);
        }
        CyEpExcelUtil.exportExcel(visitPersonList, "访客表信息", "访客表信息", VisitPerson.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysUserList);
    }
}
