package org.rcisoft.sys.visit.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.common.component.CyGlobal;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.visit.dao.VisitInfomationRepository;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitInfomationService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.sql.Wrapper;
import java.util.List;

import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by cy on 2022年11月16日 下午2:40:21.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitInfomationServiceImpl extends ServiceImpl<VisitInfomationRepository, VisitInfoMation> implements VisitInfomationService {

    @Autowired
    VisitUserRepository visitUserRepository;

    @Autowired
    SysUserRepositorys sysUserRepositorys;

    @Autowired
    VisitUserServiceImpl userService;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Autowired
    private CyGlobal global;

    @Autowired
    private PasswordEncoder passwordEncoder;

    /**
     * 保存 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(VisitInfoMation visitInfomation) {
        //增加操作
        if (CollectionUtils.isEmpty(visitInfomation.getUserList()))
            throw new CyServiceException(500, "请至少新增一名访客");
        visitInfomation.setNormal();
        int line = baseMapper.insert(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        // 添加访客信息
        visitInfomation.getUserList().forEach(u -> {
            // 判断当前访客(用户)是否已经存在
            SysUser sysUser = new SysUser();
            sysUser.setName(u.getUserName());
            sysUser.setPhone(u.getVisitPhone());
            sysUser.setUsername(u.getVisitPhone());
            sysUser.setPassword(passwordEncoder.encode(global.getResetPassword()));
            sysUser.setNormal();
            List<SysUser> sysUserList = sysUserRepositorys.querySysUsers(sysUser);
            // 不存在新增
            if (CollectionUtils.isEmpty(sysUserList)) {
                sysUserRepositorys.insert(sysUser);
            } else {
                // 存在覆盖
                sysUser.setBusinessId(sysUserList.get(0).getBusinessId());
                sysUserRepositorys.updateSysUser(sysUser);
            }
            u.setVisitId(visitInfomation.getBusinessId());
            u.setNormal();
            visitUserRepository.insert(u);
        });
        return new CyPersistModel(line);
    }

    /**
     * 删除 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(VisitInfoMation visitInfomation) {
        int line = baseMapper.realDelete(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(VisitInfoMation visitInfomation) {
        visitInfomation.setDeleted();
        int line = baseMapper.deleteById(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(VisitInfoMation visitInfomation) {
        List<VisitUser> visitUsers = visitInfomation.getUserList();

        if (visitUsers.size() > 0) {
            userService.deleteByVisitId(visitInfomation.getBusinessId());
            for (VisitUser visitUser : visitUsers) {
                userService.persist(visitUser);
            }
        }
        int line = baseMapper.updateById(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 邀访信息
     *
     * @param id
     * @return
     */
    @Override
    public VisitInfoMation findById(int id) {
        return baseMapper.queryVisitById(id);
    }

    /**
     * 分页查询 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public IPage<VisitInfoMation> findAllByPagination(CyPageInfo<VisitInfoMation> paginationUtility,
                                                      VisitInfoMation visitInfomation) {
        return baseMapper.queryVisitInfomationsPaged(paginationUtility, visitInfomation);

    }

    /**
     * 查询list 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public List<VisitInfoMation> findAll(VisitInfoMation visitInfomation) {
        List<VisitInfoMation> visitInfomationList =baseMapper.queryVisitList(visitInfomation);
        for (VisitInfoMation v: visitInfomationList) {
            String vuList = "";
            for(VisitUser vu:v.getUserList()){
                vuList = vuList+vu.getUserName()+',';
            }
            vuList = vuList.substring(0,vuList.length()-1);
            v.setUserArry(vuList);
        }
        return visitInfomationList;
    }

    /**
     * 导出邀访信息
     *
     * @return
     */
    @Override
    public List<VisitInfoMation> export(VisitInfoMation visitInfomation) {
        List<VisitInfoMation> visitInfomationList = baseMapper.queryVisitList(visitInfomation);
        for (VisitInfoMation v: visitInfomationList) {
            String vuList = "";
            for(VisitUser vu:v.getUserList()){
                vuList = vuList+vu.getUserName()+',';
            }
            vuList = vuList.substring(0,vuList.length()-1);
            v.setUserArry(vuList);
        }
        return visitInfomationList;
    }

    /**
     * 获取当前用户最新受邀记录
     *
     * @return
     */
    @Override
    public VisitInfoMation lastDetail(HttpServletRequest request) {
        // 获取当前登录用户的手机号
        String phone = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_PHONE);
        if (StringUtils.isEmpty(phone))
            return new VisitInfoMation();
        // 获取当前登录用户的受邀记录
        List<VisitInfoMation> visitInfList = baseMapper.lastDetail(phone);
        if (CollectionUtils.isEmpty(visitInfList))
            return new VisitInfoMation();
        else
            return visitInfList.get(0);
    }
}
