package org.rcisoft.sys.visit.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitUserService;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年11月16日 下午4:30:18.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitUserServiceImpl extends ServiceImpl<VisitUserRepository,VisitUser>  implements VisitUserService {


    /**
    * 保存 邀访访客信息表
    * @param visitUser
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(VisitUser visitUser){
        //增加操作
        int line = baseMapper.insert(visitUser);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            visitUser.getBusinessId()+"的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 邀访访客信息表
    * @param visitUser
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(VisitUser visitUser){
        int line = baseMapper.realDelete(visitUser);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            visitUser.getBusinessId()+"的邀访访客信息表信息");
        return new CyPersistModel(line);
    }
    /**
    * 逻辑删除 邀访访客信息表
    * @param visitUser
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(VisitUser visitUser){
        visitUser.setDeleted();
        int line = baseMapper.deleteById(visitUser);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                visitUser.getBusinessId()+"的邀访访客信息表信息");
            return new CyPersistModel(line);
        }
    /**
    * 修改 邀访访客信息表
    * @param visitUser
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(VisitUser visitUser){
        int line = baseMapper.updateById(visitUser);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ visitUser.getBusinessId()+"的邀访访客信息表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 邀访访客信息表
    * @param id
    * @return
    */
    @Override
    public VisitUser findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 邀访访客信息表
    * @param visitUser
    * @return
    */
    @Override
    public IPage<VisitUser> findAllByPagination(CyPageInfo<VisitUser> paginationUtility,
        VisitUser visitUser){
        return baseMapper.queryVisitUsersPaged(paginationUtility,visitUser);

    }
    /**
    * 查询list 邀访访客信息表
    * @param visitUser
    * @return
    */
    @Override
    public List<VisitUser> findAll(VisitUser visitUser){
        return baseMapper.queryVisitUsers(visitUser);

    }

    /**
    * 导出邀访访客信息表
    * @return
    */
    @Override
    public List<VisitUser> export(VisitUser visitUser) {
        List<VisitUser> visitUserList = baseMapper.queryVisitUsers(visitUser);
        return visitUserList;
    }

    @Override
    public int deleteByVisitId(int id) {
        return baseMapper.deleteByVisitId(id);
    }

}
